import uuid
from ast import literal_eval
from datetime import datetime, date
from dateutil.relativedelta import relativedelta
from unittest.mock import patch
from odoo import fields, release
from odoo.tests import TransactionCase


def monkey_patch(module_or_class, name):
    """ Return a method decorator to monkey-patch the given class/method. """
    def decorate(func):
        func.super = getattr(module_or_class, name, None)
        func.reset = lambda: setattr(module_or_class, name, func.super)
        if isinstance(func, classmethod):
            func.super = func.super.__func__  # get unbound classmethod
        setattr(module_or_class, name, func)
        return func
    return decorate

def eval_ping(ping_result):
    return literal_eval(ping_result)

def generate_uuid():
    return str(uuid.uuid4())

def now(**kwargs):
    timestamp = datetime.now()

    if kwargs:
        timestamp = timestamp + relativedelta(**kwargs)

    return timestamp
class FakeDatetime(datetime):
    delay = 0

    @classmethod
    def now(cls, tz=None):
        now_delay = super().now() + relativedelta(seconds=cls.delay)
        cls.delay += 1
        return now_delay


class DateBefore:
    def __init__(self, threshold):
        self._threshold = threshold

    def __eq__(self, other):
        return isinstance(other, datetime) and other <= self._threshold

    def __ne__(self, other):
        return not (self == other)

    def __repr__(self):
        return f'DateBefore({self._threshold})'


class EnterpriseDatabaseCommon(TransactionCase):

    def run(self, result=None):
        # Set default database from being hosted on a "trial" server
        with patch("odoo.addons.openerp_enterprise.models.openerp_enterprise.get_roles_from_ip",
                   return_value={'trial'}):
            super().run(result=result)

    def create_database(self, uuid=None):
        uuid = uuid if uuid else generate_uuid()

        proxy = self.env['openerp.enterprise.database']
        values = {
            'db_uuid': uuid,
            'db_name': 'db_%s' % (uuid)
        }
        return proxy.create(values)

    def send_ping(self, db_uuid, modules, db_create_date=None, errors=False, code=False, payload_update=None, fromip=None):
        if db_create_date is None:
            db_create_date = fields.Datetime.to_string(fields.Datetime.now())
        payload = {
            'dbuuid': db_uuid,
            'nbr_users': 0,
            'nbr_active_users': 0,
            'nbr_share_users': 0,
            'nbr_active_share_users': 0,
            'dbname': "test_maintenance",
            'db_create_date': db_create_date,
            'version': release.version,
            'language': 'en_US',
            'name': 'Test Maintenance',
            'phone': '+44123456789',
            'email': 'info@test-maintenance.com',
            "enterprise_code": code,
            "apps": ['mail'],
            "web_base_url": "http://localhost",
            "maintenance": {
                "version": 1,
                "modules": modules,
                "errors": errors or [],
            },
            'force_ping': True,
        }
        if payload_update:
            payload.update(payload_update)
        payload = str(payload)
        res = self.env['openerp.enterprise.ping'].sudo().ping(payload, fromip or '10.0.58.1')
        return eval_ping(res)
